/*************************************************************************/
/*************************************************************************/
/*                                                                       */
/*  File Name : EdgeList.h                                               */
/*                                                                       */
/*       Date : November  7, 1997                                        */
/* Updated at : August   22, 1998                                        */
/*                                                                       */
/*     Author : H. Timucin OZDEMIR                                       */
/*                                                                       */
/*************************************************************************/
/**
    EdgeList represents a duty which can
      1) starts from and ends at any city,
      2) no longer than the maximum duty, and
      3) not contain flying time more than the allowed limit.

    The cost of the duty is calculated as follows:
        If the total flying time is less than the paid flying time,
        then it only replects the flat cost related to any duty.
        If the total flying time is more than the paid flying time,
        then it contains flat cost as well as the cost coming from
        extra flying times as a bonus.
**/
/*************************************************************************/

#ifndef _EDGELIST_H
#define _EDGELIST_H

#include <iostream.h>
#include <gVector.h> 

class EdgeList;
typedef EdgeList* EdgeListPtr;

#include "Edge.h" 
#include "City.h" 
#include "AirScheduleProblem.h" 

//typedef Edge* EdgePtr;
//typedef EdgeList* EdgeListPtr;

class EdgeList : public gVector<EdgePtr> {

  public :

     EdgeList() ;
     EdgeList(int _size) ;
     EdgeList(EdgePtr edge); 
     EdgeList(EdgeList& RHS); 

     float getCost(float CostPerMinute,float CostPerOvernight) ;
     getNumberOfOvernightStays() ;
     getTotalFlyingTime() ;
     getTotalTimeSpan() const { return (getEndDate()-getStartDate()); };
     void addEdge(EdgePtr edge);
     addEdgeFront(EdgePtr edge) ;

     doesContainNodeId(int nodeId) const;
     void splitListFromNode(int nodeId,EdgeList& first, EdgeList& second);

     Edge *getFirstEdge() const; 
     Edge *getLastEdge() const ;
     Edge *getNthEdge(unsigned indx) const ;

     int   getNumberOfEdges() const ;
     getStartCityIndex() const;
     getDestinationCityIndex() const;

     doesStartFromBaseCity() const;
     doesEndAtBaseCity() const;

     inline isPath() const 
     { return (getStartCityIndex()!=getDestinationCityIndex()); }

     inline isBaseCyle() const 
     { return (!isPath()&&doesStartFromBaseCity()); }

     inline isCyle() const { return (!isPath()); }

     canFollow(EdgeList& RHS) ;
     canFollow(EdgePtr RHS) ;
     canPrecede(EdgeList& RHS) ;

     void removeEdgesStartFrom(int);
     void removeEdgesEndAt(int);
     void removeFromTo(int,int);

     Date getStartDate() const;
     Date getEndDate() const;

     operator==(const EdgeList&);
     EdgeList operator=(const EdgeList&);

     friend ostream& operator<<(ostream& os, const EdgeList& RHS);

     isValid(AirScheduleProblem*);

     float getDCost( AirScheduleProblem*) ;

     inline void  clear() { myReset(); reset(); }
     float getLastDutyLength( AirScheduleProblem*) ;
     float getFirstDutyLength( AirScheduleProblem*) ;

     canMerge( EdgeList& eList, AirScheduleProblem* problem);
     mergeWith( EdgeList& eList );

     float getCostPerUncoveredMinute(AirScheduleProblem* myProblem, char* covered);
     getNumberOfDifferences(EdgeListPtr eList);


   private:

     void nicePrint(ostream&) const;
     int   costCalculated;
     float ecost;
     float dCost;
     int   totalFlyingTime;
     int   totalOverNights;
     int   timeCalculated;
     int   numberOfDuties;
     char  dCostCalculated; 
     float lastDutyLength;
     float firstDutyLength;

     void CalculateCost(float CostPerMinute,float CostPerOvernight) ;
     void CalculateTime();
     void myReset();

}; // end of Class EdgeList

#endif
